import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.concurrent.Semaphore;

/**
 * This Dealer class runs a game with a number of players and a user at the
 * keyboard.  The object of the game is for the user at the keyboard to
 * guess numbers that the players will find acceptable.  As each player
 * gets a number that it likes, it will leave the table (exit its run method).
 */
public class Dealer {

  /**
   * Run the game by starting the players up and then asking the user for
   * numbers to supply to the players.
   * @param args ignored
   */
  public static void main(String[] args) throws IOException {
    
    System.out.println("Type your integer guess when ready.");
    
    BufferedReader cIn = new BufferedReader(new InputStreamReader(System.in));

    // The board is a little data holder.  The players and the dealer use
    // it for data transfer and synchronization purposes
    
    GameBoard board = new GameBoard();
    
    // Maximum number of active players is limited by a counting Semaphore
    
    int seatCount = 3;
    Semaphore ticketManager = new Semaphore(seatCount, true);

    // Each player is a different thread
    
    BasicPlayer[] pool = new BasicPlayer[] {
        new OddNumbers(ticketManager, board),
        new OddNumbers(ticketManager, board),
        new NegativeNumbers(ticketManager, board),
        new OddNumbers(ticketManager, board),
        new OddNumbers(ticketManager, board),
        new OddNumbers(ticketManager, board) };

    char letter = 'A';
    for (int i = 0; i < pool.length; i++) {
      pool[i].setName(Character.toString(letter++));
      pool[i].start();
    }
    
    // Get numbers from the user at the keyboard and
    // let the players take a look at them.

    String line = cIn.readLine();
    while (line != null && line.length() != 0) {
      int newGuess = Integer.parseInt(line);
      synchronized (board) {
        board.setGuess(newGuess);
        board.notifyAll();
      }
      line = cIn.readLine();
    }
    System.out.println("Game over.");
    System.exit(0);
  }
}
